# Copyright 2018 Canonical Limited.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import mock
import os
import unittest

import charmhelpers.contrib.openstack.vaultlocker as vaultlocker

from .test_os_contexts import TestDB


INCOMPLETE_RELATION = {
    'secrets-storage:1': {
        'vault/0': {}
    }
}

COMPLETE_RELATION = {
    'secrets-storage:1': {
        'vault/0': {
            'vault_url': json.dumps('http://vault:8200'),
            'test-service/0_role_id': json.dumps('test-role-from-vault'),
            'test-service/0_token':
                json.dumps('00c9a9ab-c523-459d-a250-2ce8f0877c03'),
        }
    }
}

COMPLETE_WITH_CA_RELATION = {
    'secrets-storage:1': {
        'vault/0': {
            'vault_url': json.dumps('http://vault:8200'),
            'test-service/0_role_id': json.dumps('test-role-from-vault'),
            'test-service/0_token':
                json.dumps('00c9a9ab-c523-459d-a250-2ce8f0877c03'),
            'vault_ca': json.dumps('test-ca-data'),
        }
    }
}


class VaultLockerTestCase(unittest.TestCase):

    to_patch = [
        'hookenv',
        'templating',
        'alternatives',
        'host',
        'unitdata',
    ]

    _target_path = '/var/lib/charm/test-service/vaultlocker.conf'

    def setUp(self):
        for m in self.to_patch:
            setattr(self, m, self._patch(m))
        self.hookenv.service_name.return_value = 'test-service'
        self.hookenv.local_unit.return_value = 'test-service/0'
        self.db = TestDB()
        self.unitdata.kv.return_value = self.db

    def _patch(self, target):
        _m = mock.patch.object(vaultlocker, target)
        _mock = _m.start()
        self.addCleanup(_m.stop)
        return _mock

    def test_write_vl_config(self):
        ctxt = {'test': 'data'}
        vaultlocker.write_vaultlocker_conf(context=ctxt)
        self.hookenv.service_name.assert_called_once_with()
        self.host.mkdir.assert_called_once_with(
            os.path.dirname(self._target_path),
            perms=0o700
        )
        self.templating.render.assert_called_once_with(
            source='vaultlocker.conf.j2',
            target=self._target_path,
            context=ctxt,
            perms=0o600,
        )
        self.alternatives.install_alternative.assert_called_once_with(
            'vaultlocker.conf',
            '/etc/vaultlocker/vaultlocker.conf',
            self._target_path,
            100
        )

    def test_write_vl_config_priority(self):
        ctxt = {'test': 'data'}
        vaultlocker.write_vaultlocker_conf(context=ctxt, priority=200)
        self.hookenv.service_name.assert_called_once_with()
        self.host.mkdir.assert_called_once_with(
            os.path.dirname(self._target_path),
            perms=0o700
        )
        self.templating.render.assert_called_once_with(
            source='vaultlocker.conf.j2',
            target=self._target_path,
            context=ctxt,
            perms=0o600,
        )
        self.alternatives.install_alternative.assert_called_once_with(
            'vaultlocker.conf',
            '/etc/vaultlocker/vaultlocker.conf',
            self._target_path,
            200
        )

    def _setup_relation(self, relation):
        self.hookenv.relation_ids.side_effect = (
            lambda _: relation.keys()
        )
        self.hookenv.related_units.side_effect = (
            lambda rid: relation[rid].keys()
        )
        self.hookenv.relation_get.side_effect = (
            lambda unit, rid:
                relation[rid][unit]
        )

    def test_context_incomplete(self):
        self._setup_relation(INCOMPLETE_RELATION)
        context = vaultlocker.VaultKVContext('charm-test')
        self.assertEqual(context(), {})
        self.hookenv.relation_ids.assert_called_with('secrets-storage')
        self.assertFalse(vaultlocker.vault_relation_complete())

    @mock.patch.object(vaultlocker, 'retrieve_secret_id')
    def test_context_complete(self, retrieve_secret_id):
        self._setup_relation(COMPLETE_RELATION)
        context = vaultlocker.VaultKVContext('charm-test')
        retrieve_secret_id.return_value = 'a3551c8d-0147-4cb6-afc6-efb3db2fccb2'
        self.assertEqual(context(),
                         {'role_id': 'test-role-from-vault',
                          'secret_backend': 'charm-test',
                          'secret_id': 'a3551c8d-0147-4cb6-afc6-efb3db2fccb2',
                          'vault_url': 'http://vault:8200'})
        self.hookenv.relation_ids.assert_called_with('secrets-storage')
        self.assertTrue(vaultlocker.vault_relation_complete())
        retrieve_secret_id.assert_called_once_with(
            url='http://vault:8200',
            token='00c9a9ab-c523-459d-a250-2ce8f0877c03'
        )

    @mock.patch.object(vaultlocker, 'retrieve_secret_id')
    def test_context_complete_cached_secret_id(self, retrieve_secret_id):
        self._setup_relation(COMPLETE_RELATION)
        context = vaultlocker.VaultKVContext('charm-test')
        self.db.set('last-token', '00c9a9ab-c523-459d-a250-2ce8f0877c03')
        self.db.set('secret-id', '5502fd27-059b-4b0a-91b2-eaff40b6a112')
        self.assertEqual(context(),
                         {'role_id': 'test-role-from-vault',
                          'secret_backend': 'charm-test',
                          'secret_id': '5502fd27-059b-4b0a-91b2-eaff40b6a112',
                          'vault_url': 'http://vault:8200'})
        self.hookenv.relation_ids.assert_called_with('secrets-storage')
        self.assertTrue(vaultlocker.vault_relation_complete())
        retrieve_secret_id.assert_not_called()

    @mock.patch.object(vaultlocker, 'retrieve_secret_id')
    def test_context_complete_with_ca(self, retrieve_secret_id):
        self._setup_relation(COMPLETE_WITH_CA_RELATION)
        context = vaultlocker.VaultKVContext('charm-test')
        retrieve_secret_id.return_value = 'a3551c8d-0147-4cb6-afc6-efb3db2fccb2'
        self.assertEqual(context(),
                         {'role_id': 'test-role-from-vault',
                          'secret_backend': 'charm-test',
                          'secret_id': 'a3551c8d-0147-4cb6-afc6-efb3db2fccb2',
                          'vault_url': 'http://vault:8200',
                          'vault_ca': 'test-ca-data'})
        self.hookenv.relation_ids.assert_called_with('secrets-storage')
        self.assertTrue(vaultlocker.vault_relation_complete())
        retrieve_secret_id.assert_called_once_with(
            url='http://vault:8200',
            token='00c9a9ab-c523-459d-a250-2ce8f0877c03'
        )



# Copyright 2014-2015 Canonical Limited.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import subprocess
import time
import os
from distutils.spawn import find_executable

from charmhelpers.core.hookenv import (
    in_relation_hook,
    relation_ids,
    relation_set,
    relation_get,
)


def action_set(key, val):
    if find_executable('action-set'):
        action_cmd = ['action-set']

        if isinstance(val, dict):
            for k, v in iter(val.items()):
                action_set('%s.%s' % (key, k), v)
            return True

        action_cmd.append('%s=%s' % (key, val))
        subprocess.check_call(action_cmd)
        return True
    return False


class Benchmark():
    """
    Helper class for the `benchmark` interface.

    :param list actions: Define the actions that are also benchmarks

    From inside the benchmark-relation-changed hook, you would
    Benchmark(['memory', 'cpu', 'disk', 'smoke', 'custom'])

    Examples:

        siege = Benchmark(['siege'])
        siege.start()
        [... run siege ...]
        # The higher the score, the better the benchmark
        siege.set_composite_score(16.70, 'trans/sec', 'desc')
        siege.finish()


    """

    BENCHMARK_CONF = '/etc/benchmark.conf'  # Replaced in testing

    required_keys = [
        'hostname',
        'port',
        'graphite_port',
        'graphite_endpoint',
        'api_port'
    ]

    def __init__(self, benchmarks=None):
        if in_relation_hook():
            if benchmarks is not None:
                for rid in sorted(relation_ids('benchmark')):
                    relation_set(relation_id=rid, relation_settings={
                        'benchmarks': ",".join(benchmarks)
                    })

            # Check the relation data
            config = {}
            for key in self.required_keys:
                val = relation_get(key)
                if val is not None:
                    config[key] = val
                else:
                    # We don't have all of the required keys
                    config = {}
                    break

            if len(config):
                with open(self.BENCHMARK_CONF, 'w') as f:
                    for key, val in iter(config.items()):
                        f.write("%s=%s\n" % (key, val))

    @staticmethod
    def start():
        action_set('meta.start', time.strftime('%Y-%m-%dT%H:%M:%SZ'))

        """
        If the collectd charm is also installed, tell it to send a snapshot
        of the current profile data.
        """
        COLLECT_PROFILE_DATA = '/usr/local/bin/collect-profile-data'
        if os.path.exists(COLLECT_PROFILE_DATA):
            subprocess.check_output([COLLECT_PROFILE_DATA])

    @staticmethod
    def finish():
        action_set('meta.stop', time.strftime('%Y-%m-%dT%H:%M:%SZ'))

    @staticmethod
    def set_composite_score(value, units, direction='asc'):
        """
        Set the composite score for a benchmark run. This is a single number
        representative of the benchmark results. This could be the most
        important metric, or an amalgamation of metric scores.
        """
        return action_set(
            "meta.composite",
            {'value': value, 'units': units, 'direction': direction}
        )



from mock import patch
from testtools import TestCase

from charmhelpers.contrib.templating.pyformat import render
from charmhelpers.core import hookenv


class PyFormatTest(TestCase):
    @patch.object(hookenv, 'execution_environment')
    def test_renders_using_environment(self, execution_environment):
        execution_environment.return_value = {
            'foo': 'FOO',
        }

        self.assertEqual(render('foo is {foo}'), 'foo is FOO')

    @patch.object(hookenv, 'execution_environment')
    def test_extra_overrides(self, execution_environment):
        execution_environment.return_value = {
            'foo': 'FOO',
        }

        extra = {'foo': 'BAR'}

        self.assertEqual(render('foo is {foo}', extra=extra), 'foo is BAR')

    @patch.object(hookenv, 'execution_environment')
    def test_kwargs_overrides(self, execution_environment):
        execution_environment.return_value = {
            'foo': 'FOO',
        }

        extra = {'foo': 'BAR'}

        self.assertEqual(
            render('foo is {foo}', extra=extra, foo='BAZ'), 'foo is BAZ')
